@@ -21,7 +21,6 @@
 
 #include <language/codecompletion/codecompletioncontext.h>
 #include <language/duchain/duchainpointer.h>
-#include <kdev-pg-memory-pool.h>
 #include <QStack>
 
 #include <QObject>
@@ -51,10 +50,10 @@ typedef QPair<Declaration*, int> DeclarationDepthPair;
  **/
 class IncludeSearchTarget {
 public:
-    IncludeSearchTarget(KUrl d_, QStringList r_) : directory(d_), remainingIdentifiers(r_) {
-        directory.cleanPath();
+    IncludeSearchTarget(QUrl d_, QStringList r_) : directory(d_), remainingIdentifiers(r_) {
+        directory.setPath(QDir::cleanPath(directory.path()));
     };
-    KUrl directory;
+    QUrl directory;
     QStringList remainingIdentifiers;
 };
 
@@ -73,22 +72,30 @@ public:
         FunctionCallCompletion,
         InheritanceCompletion,
         RaiseExceptionCompletion,
-        GeneratorVariableCompletion
+        GeneratorVariableCompletion,
+        StringFormattingCompletion
     };
     
     enum ItemTypeHint {
         NoHint,
-        IterableRequested
+        IterableRequested, // < the requested item should be iterable, e.g. a list
+        ClassTypeRequested // < the requested item should be a class type
     };
     
     PythonCodeCompletionContext(DUContextPointer context, const QString& text, const QString& followingText,
                                 const KDevelop::CursorInRevision& position,
                                 int depth, const PythonCodeCompletionWorker* parent);
+
+    CompletionContextType completionContextType();
+
     ItemTypeHint itemTypeHint();
+
     /**
      * @brief The "interface method" which returns all the completion items to kdevelop.
      **/
     virtual QList< KDevelop::CompletionTreeItemPointer > completionItems(bool& abort, bool fullCompletion = true);
+
+    virtual QList< CompletionTreeElementPointer > ungroupedElements();
     
     /**
      * @brief Get all possible items matching the specified dot-seperated search string.
@@ -132,6 +139,22 @@ private:
     void summonParentForEventualCall(TokenList tokens, const QString& text);
     /// Get possible "add import..." items for an expression.
     QList< CompletionTreeItemPointer > getMissingIncludeItems(QString forString);
+    void eventuallyAddGroup(QString name, int priority, QList<CompletionTreeItemPointer> items);
+
+private:
+    /// Item generating functions
+    using ItemList = QList<CompletionTreeItemPointer>;
+    ItemList shebangItems();
+    ItemList generatorItems();
+    ItemList functionCallItems();
+    ItemList defineItems();
+    ItemList raiseItems();
+    ItemList importFileItems();
+    ItemList inheritanceItems();
+    ItemList memberAccessItems();
+    ItemList stringFormattingItems();
+    ItemList keywordItems();
+    ItemList classMemberInitItems();
 
 private:
     CompletionContextType m_operation;
@@ -140,7 +163,7 @@ private:
     int m_maxFolderScanDepth;
     QStringList m_searchingForModule;
     QString m_searchImportItemsInModule;
-    KUrl m_workingOnDocument;
+    QUrl m_workingOnDocument;
     
     CodeCompletionContext* m_child;
     
@@ -156,6 +179,8 @@ private:
     QString m_matchAgainst;
 
     bool m_fullCompletion;
+
+    QList<CompletionTreeElementPointer> m_storedGroups;
 };
 
 }
