@@ -19,8 +19,7 @@
 
 #include <QTimer>
 #include <QApplication>
-#include <KDebug>
-#include <KStandardDirs>
+
 #include <KLocalizedString>
 #include <signal.h>
 
@@ -34,6 +33,10 @@
 #include "variable.h"
 #include "breakpointcontroller.h"
 
+#include <QDebug>
+#include <QStandardPaths>
+#include "debuggerdebug.h"
+
 using namespace KDevelop;
 
 static QByteArray debuggerPrompt = "__KDEVPYTHON_DEBUGGER_PROMPT";
@@ -42,29 +45,37 @@ static QByteArray debuggerOutputEnd = "<<<__KDEVPYTHON_END___DEBUGGER_OUTPUT";
 
 namespace Python {
 
-KDevelop::IFrameStackModel* DebugSession::createFrameStackModel()
-{
-    return new PdbFrameStackModel(this);
-}
-
-DebugSession::DebugSession(QStringList program, const KUrl &workingDirectory) :
+DebugSession::DebugSession(QStringList program, const QUrl &workingDirectory) :
     IDebugSession()
+    , m_breakpointController(nullptr)
+    , m_variableController(nullptr)
+    , m_frameStackModel(nullptr)
     , m_workingDirectory(workingDirectory)
     , m_nextNotifyMethod(0)
     , m_inDebuggerData(0)
 {
-    kDebug() << "creating debug session";
-    m_variableController = new Python::VariableController(this);
-    m_breakpointController = new Python::BreakpointController(this);
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "creating debug session";
     m_program = program;
+    m_breakpointController = new Python::BreakpointController(this);
     m_variableController = new VariableController(this);
+    m_frameStackModel = new PdbFrameStackModel(this);
 }
 
-IVariableController* DebugSession::variableController()
+IBreakpointController* DebugSession::breakpointController() const
+{
+    return m_breakpointController;
+}
+
+IVariableController* DebugSession::variableController() const
 {
     return m_variableController;
 }
 
+IFrameStackModel* DebugSession::frameStackModel() const
+{
+    return m_frameStackModel;
+}
+
 void DebugSession::start()
 {
     setState(StartingState);
@@ -79,8 +90,10 @@ void DebugSession::start()
     connect(this, SIGNAL(commandAdded()), SLOT(checkCommandQueue()));
     m_debuggerProcess->start();
     m_debuggerProcess->waitForStarted();
+    auto dir = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
+                                      "kdevpythonsupport/debugger/", QStandardPaths::LocateDirectory);
     InternalPdbCommand* path = new InternalPdbCommand(0, 0,
-        "import sys; sys.path.append('"+KStandardDirs::locate("data", "kdevpythonsupport/debugger/")+"')\n");
+        "import sys; sys.path.append('"+dir+"')\n");
     InternalPdbCommand* cmd = new InternalPdbCommand(0, 0, "import __kdevpython_debugger_utils\n");
     addCommand(path);
     addCommand(cmd);
@@ -107,7 +120,7 @@ QStringList byteArrayToStringList(const QByteArray& r) {
 void DebugSession::dataAvailable()
 {
     QByteArray data = m_debuggerProcess->readAllStandardOutput();
-    kDebug() << data.length() << "bytes of data available";
+    qCDebug(KDEV_PYTHON_DEBUGGER) << data.length() << "bytes of data available";
     
     // remove pointless state changes
     data.replace(debuggerOutputBegin+debuggerOutputEnd, "");
@@ -131,7 +144,7 @@ void DebugSession::dataAvailable()
         nextChangeAt = atLastChange ? len : qMin(nextChangeAt, len);
 
         
-        kDebug() << data;
+        qCDebug(KDEV_PYTHON_DEBUGGER) << data;
         Q_ASSERT(m_inDebuggerData == 0 || m_inDebuggerData == 1);
         
         if ( m_inDebuggerData == 1 ) {
@@ -181,7 +194,7 @@ void DebugSession::dataAvailable()
         else {
             notifyNext();
             if ( m_commandQueue.isEmpty() ) {
-                kDebug() << "Changing state to PausedState";
+                qCDebug(KDEV_PYTHON_DEBUGGER) << "Changing state to PausedState";
                 setState(PausedState);
             }
         }
@@ -195,22 +208,22 @@ void DebugSession::dataAvailable()
     }
 }
 
-void DebugSession::setNotifyNext(QWeakPointer<QObject> object, const char* method)
+void DebugSession::setNotifyNext(QPointer<QObject> object, const char* method)
 {
-    kDebug() << "set notify next:" << object << method;
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "set notify next:" << object << method;
     m_nextNotifyObject = object;
     m_nextNotifyMethod = method;
 }
 
 void DebugSession::notifyNext()
 {
-    QSharedPointer<QObject> lock = m_nextNotifyObject.toStrongRef();
-    kDebug() << "notify next:" << m_nextNotifyObject << m_nextNotifyObject.data() << this;
-    if ( m_nextNotifyMethod and m_nextNotifyObject ) {
-        QMetaObject::invokeMethod(m_nextNotifyObject.data(), m_nextNotifyMethod, Qt::DirectConnection, Q_ARG(QByteArray, m_buffer));
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "notify next:" << m_nextNotifyObject << this;
+    if ( m_nextNotifyMethod && m_nextNotifyObject ) {
+        QMetaObject::invokeMethod(m_nextNotifyObject.data(), m_nextNotifyMethod,
+                                  Qt::DirectConnection, Q_ARG(QByteArray, m_buffer));
     }
     else {
-        kDebug() << "notify called, but nothing to notify!";
+        qCDebug(KDEV_PYTHON_DEBUGGER) << "notify called, but nothing to notify!";
     }
     m_buffer.clear();
     m_nextNotifyMethod = 0;
@@ -219,9 +232,9 @@ void DebugSession::notifyNext()
 
 void DebugSession::processNextCommand()
 {
-    kDebug() << "processing next debugger command in queue";
-    if ( m_processBusy or m_state == EndedState ) {
-        kDebug() << "process is busy or ended, aborting";
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "processing next debugger command in queue";
+    if ( m_processBusy || m_state == EndedState ) {
+        qCDebug(KDEV_PYTHON_DEBUGGER) << "process is busy or ended, aborting";
         return;
     }
     m_processBusy = true;
@@ -233,7 +246,7 @@ void DebugSession::processNextCommand()
     m_commandQueue.removeFirst();
     setNotifyNext(cmd->notifyObject(), cmd->notifyMethod());
     cmd->run(this);
-    kDebug() << "command executed, deleting it.";
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "command executed, deleting it.";
     delete cmd;
     if ( ! m_commandQueue.isEmpty() ) {
         processNextCommand();
@@ -242,7 +255,7 @@ void DebugSession::processNextCommand()
 
 void DebugSession::setState(DebuggerState state)
 {
-    kDebug() << "Setting state to" << state;
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "Setting state to" << state;
     
     if ( state == m_state ) {
         return;
@@ -262,14 +275,14 @@ void DebugSession::setState(DebuggerState state)
         }
     }
     
-    kDebug() << "debugger state changed to" << m_state;
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "debugger state changed to" << m_state;
     raiseEvent(program_state_changed);
     emit stateChanged(m_state);
 }
 
 void DebugSession::write(const QByteArray& cmd)
 {
-    kDebug() << " >>> WRITE:" << cmd;
+    qCDebug(KDEV_PYTHON_DEBUGGER) << " >>> WRITE:" << cmd;
     m_debuggerProcess->write(cmd);
 }
 
@@ -305,7 +318,7 @@ void DebugSession::jumpToCursor()
         KTextEditor::Cursor cursor = doc->cursorPosition();
         if ( cursor.isValid() ) {
             // TODO disable all other breakpoints
-            addSimpleUserCommand(QString("jump " + QString::number(cursor.line() + 1)).toAscii());
+            addSimpleUserCommand(QString("jump " + QString::number(cursor.line() + 1)).toUtf8());
         }
     }
 }
@@ -342,7 +355,7 @@ void DebugSession::addCommand(PdbCommand* cmd)
     if ( m_state == EndedState || m_state == StoppingState ) {
         return;
     }
-    kDebug() << " +++  adding command to queue:" << cmd;
+    qCDebug(KDEV_PYTHON_DEBUGGER) << " +++  adding command to queue:" << cmd;
     m_commandQueue.append(cmd);
     if ( cmd->type() == PdbCommand::UserType ) {
         // this is queued and will run after the command is executed.
@@ -353,7 +366,7 @@ void DebugSession::addCommand(PdbCommand* cmd)
 
 void DebugSession::checkCommandQueue()
 {
-    kDebug() << "items in queue:" << m_commandQueue.length();
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "items in queue:" << m_commandQueue.length();
     if ( m_commandQueue.isEmpty() ) {
         return;
     }
@@ -386,9 +399,9 @@ void DebugSession::runImmediately(const QString& cmd)
     if ( state() == ActiveState ) {
         m_nextNotifyMethod = 0;
         m_nextNotifyObject.clear(); // TODO is this correct?
-        kDebug() << "interrupting debugger";
+        qCDebug(KDEV_PYTHON_DEBUGGER) << "interrupting debugger";
         kill(m_debuggerProcess->pid(), SIGINT);
-        write(cmd.toAscii());
+        write(cmd.toUtf8());
         write("continue\n");
         updateLocation();
     }
@@ -400,21 +413,22 @@ void DebugSession::runImmediately(const QString& cmd)
 void DebugSession::addBreakpoint(Breakpoint* bp)
 {
     QString location = bp->url().path() + ":" + QString::number(bp->line() + 1);
-    kDebug() << "adding breakpoint" << location;
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "adding breakpoint" << location;
     runImmediately("break " + location + '\n');
 }
 
 void DebugSession::removeBreakpoint(Breakpoint* bp)
 {
     QString location = bp->url().path() + ":" + QString::number(bp->line() + 1);
-    kDebug() << "deleting breakpoint" << location;
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "deleting breakpoint" << location;
     runImmediately("clear " + location + '\n');
 }
 
 void DebugSession::createVariable(Python::Variable* variable, QObject* callback, const char* callbackMethod)
 {
-    kDebug() << "asked to create variable";
-    InternalPdbCommand* cmd = new InternalPdbCommand(variable, "dataFetched", ("print " + variable->expression() + '\n').toAscii());
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "asked to create variable";
+    InternalPdbCommand* cmd = new InternalPdbCommand(variable, "dataFetched",
+                                                     ("print(" + variable->expression() + ")\n").toUtf8());
     variable->m_notifyCreated = callback;
     variable->m_notifyCreatedMethod = callbackMethod;
     addCommand(cmd);
@@ -427,13 +441,13 @@ void DebugSession::clearOutputBuffer()
 
 void DebugSession::updateLocation()
 {
-    kDebug() << "updating location";
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "updating location";
     InternalPdbCommand* cmd = new InternalPdbCommand(this, "locationUpdateReady", "where\n");
     addCommand(cmd);
 }
 
 void DebugSession::locationUpdateReady(QByteArray data) {
-    kDebug() << "Got where information: " << data;
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "Got where information: " << data;
     QList<QByteArray> lines = data.split('\n');
     if ( lines.length() >= 3 ) {
         lines.removeLast(); // prompt
@@ -443,8 +457,8 @@ void DebugSession::locationUpdateReady(QByteArray data) {
         QRegExp m("^> (/.*\\.py)\\((\\d*)\\).*$");
         m.setMinimal(true);
         m.exactMatch(where);
-        setCurrentPosition(KUrl(m.capturedTexts().at(1)), m.capturedTexts().at(2).toInt() - 1 , "<unknown>");
-        kDebug() << "New position: " << m.capturedTexts().at(1) << m.capturedTexts().at(2).toInt() - 1 << m.capturedTexts() << where;
+        setCurrentPosition(QUrl::fromLocalFile(m.capturedTexts().at(1)), m.capturedTexts().at(2).toInt() - 1 , "<unknown>");
+        qCDebug(KDEV_PYTHON_DEBUGGER) << "New position: " << m.capturedTexts().at(1) << m.capturedTexts().at(2).toInt() - 1 << m.capturedTexts() << where;
     }
 }
 
@@ -460,7 +474,7 @@ void DebugSession::stopDebugger()
     m_commandQueue.clear();
     m_nextNotifyMethod = 0;
     m_nextNotifyObject.clear();
-    kDebug() << "killed debugger";
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "killed debugger";
     setState(IDebugSession::EndedState);
 }
 
@@ -487,4 +501,3 @@ KDevelop::IDebugSession::DebuggerState DebugSession::state() const
 
 }
 
-#include "debugsession.moc"
