@@ -1,6 +1,7 @@
 /***************************************************************************
  *   This file is part of KDevelop                                         *
  *   Copyright 2007 Andreas Pakulat <apaku@gmx.de>                         *
+ *   Copyright 2012 Patrick Spendrin <ps_ml@gmx.de>                        *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -18,17 +19,15 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
-// The Python 2.7 Language Reference was used as basis for this AST
+// The Python 3.4 Language Reference was used as basis for this AST
 
 #ifndef PYTHON_AST_H
 #define PYTHON_AST_H
 
 #include <QList>
-#include <QMap>
 #include <QString>
-#include <QStringList>
-#include <QPair>
-
+#include <QSharedPointer>
+#include <KTextEditor/Range>
 #include "parserexport.h"
 
 namespace KDevelop
@@ -40,8 +39,8 @@ namespace Python {
     class StatementAst;
     class FunctionDefinitionAst;
     class AssignmentAst;
-    class PrintAst;
     class PassAst;
+    class NonlocalAst;
     class ExpressionAst;
     class NameAst;
     class CallAst;
@@ -69,14 +68,13 @@ class KDEVPYTHONPARSER_EXPORT Ast
 public:
     enum AstType
     {
+        StatementAstType,
         FunctionDefinitionAstType,
         AssignmentAstType,
-        PrintAstType,
         PassAstType,
-        NameAstType,
-        CallAstType,
-        AttributeAstType,
+        NonlocalAstType,
         ArgumentsAstType,
+        ArgAstType,
         KeywordAstType,
         ClassDefinitionAstType,
         ReturnAstType,
@@ -85,23 +83,25 @@ public:
         WhileAstType,
         IfAstType,
         WithAstType,
+        WithItemAstType,
         RaiseAstType,
-        TryExceptAstType,
-        TryFinallyAstType,
+        TryAstType,
         ImportAstType,
         ImportFromAstType,
-        ExecAstType,
         GlobalAstType,
         BreakAstType,
         ContinueAstType,
         AssertionAstType,
         AugmentedAssignmentAstType,
-        DictionaryComprehensionAstType,
+        LastStatementType,
+        ExpressionAstType, // everything below is an expression
+        AwaitAstType,
+        NameAstType,
+        NameConstantAstType,
+        CallAstType,
+        AttributeAstType,
         ExtendedSliceAstType,
-        CodeAstType,
-        StatementAstType,
-        ExpressionAstType,
-        
+        DictionaryComprehensionAstType,
         BooleanOperationAstType,
         BinaryOperationAstType,
         UnaryOperationAstType,
@@ -114,22 +114,26 @@ public:
         GeneratorExpressionAstType,
         YieldAstType,
         CompareAstType,
-        ReprAstType,
         NumberAstType,
         StringAstType,
+        BytesAstType,
         SubscriptAstType,
+        StarredAstType,
         ListAstType,
         TupleAstType,
-        
+        YieldFromAstType,
+        ComprehensionAstType,
+
         SliceAstType,
         EllipsisAstType,
         IndexAstType,
-        
-        ComprehensionAstType,
+        LastExpressionType, // keep this at the end of the expr ast list
+
+        CodeAstType,
         ExceptionHandlerAstType,
         AliasAstType, // for imports
-        
-        IdentifierAstType
+        IdentifierAstType,
+        LastAstType // the largest one, not valid!
     };
     
     enum BooleanOperationTypes {
@@ -142,6 +146,7 @@ public:
         OperatorAdd = 1,
         OperatorSub,
         OperatorMult,
+        OperatorMatMult,
         OperatorDiv,
         OperatorMod,
         OperatorPow,
@@ -178,15 +183,46 @@ public:
 
     Ast(Ast* parent, AstType type);
     Ast();
-    Ast* parent;
+    Ast* parent = nullptr;
     AstType astType;
-    
+
+    bool isExpression() const {
+        return astType >= ExpressionAstType && astType <= LastExpressionType;
+    }
+
     void copyRange(const Ast* other) {
         startCol = other->startCol;
         endCol = other->endCol;
         startLine = other->startLine;
         endLine = other->endLine;
     }
+    
+    bool appearsBefore(const Ast* other) {
+        return startLine < other->startLine || ( startLine == other->startLine && startCol < other->startCol );
+    }
+    
+    const KTextEditor::Range range() const {
+        return KTextEditor::Range(startLine, startCol, endLine, endCol);
+    };
+    
+    const KTextEditor::Cursor start() const {
+        return {startLine, startCol};
+    }
+    
+    const KTextEditor::Cursor end() const {
+        return {endLine, endCol};
+    }
+
+    bool isChildOf(Ast* other) const {
+        const Ast* parent = this;
+        while ( parent ) {
+            if ( parent == other ) {
+                return true;
+            }
+            parent = parent->parent;
+        }
+        return false;
+    }
 
     int startCol;
     int startLine;
@@ -230,6 +266,8 @@ public:
 class KDEVPYTHONPARSER_EXPORT CodeAst : public Ast {
 public:
     CodeAst();
+    ~CodeAst();
+    typedef QSharedPointer<CodeAst> Ptr;
     QList<Ast*> body;
     Identifier* name; // module name
 };
@@ -247,6 +285,8 @@ public:
     ArgumentsAst* arguments;
     QList<ExpressionAst*> decorators;
     QList<Ast*> body;
+    ExpressionAst* returns;
+    bool async;
 };
 
 class KDEVPYTHONPARSER_EXPORT ClassDefinitionAst : public StatementAst {
@@ -310,12 +350,18 @@ public:
     QList<Ast*> orelse;
 };
 
+class KDEVPYTHONPARSER_EXPORT WithItemAst : public Ast {
+public:
+    WithItemAst(Ast* parent);
+    ExpressionAst* contextExpression;
+    NameAst* optionalVars;
+};
+
 class KDEVPYTHONPARSER_EXPORT WithAst : public StatementAst {
 public:
     WithAst(Ast* parent);
-    ExpressionAst* contextExpression;
-    ExpressionAst* optionalVars;
     QList<Ast*> body;
+    QList<WithItemAst*> items;
 };
 
 class KDEVPYTHONPARSER_EXPORT RaiseAst : public StatementAst {
@@ -325,19 +371,13 @@ public:
     // TODO check what the other things in the grammar actually are and add them
 };
 
-class KDEVPYTHONPARSER_EXPORT TryExceptAst : public StatementAst {
+class KDEVPYTHONPARSER_EXPORT TryAst : public StatementAst {
 public:
-    TryExceptAst(Ast* parent);
+    TryAst(Ast* parent);
     QList<Ast*> body;
     QList<ExceptionHandlerAst*> handlers;
     QList<Ast*> orelse;
-};
-
-class KDEVPYTHONPARSER_EXPORT TryFinallyAst : public StatementAst {
-public:
-    TryFinallyAst(Ast* parent);
-    QList<Ast*> body;
-    QList<Ast*> finalbody;
+    QList<Ast*> finally;
 };
 
 class KDEVPYTHONPARSER_EXPORT AssertionAst : public StatementAst {
@@ -361,14 +401,6 @@ public:
     int level;
 };
 
-class KDEVPYTHONPARSER_EXPORT ExecAst : public StatementAst {
-public:
-    ExecAst(Ast* parent);
-    ExpressionAst* body;
-    ExpressionAst* globals;
-    ExpressionAst* locals;
-};
-
 class KDEVPYTHONPARSER_EXPORT GlobalAst : public StatementAst {
 public:
     GlobalAst(Ast* parent);
@@ -387,19 +419,16 @@ public:
     ContinueAst(Ast* parent);
 };
 
-class KDEVPYTHONPARSER_EXPORT PrintAst : public StatementAst {
-public:
-    PrintAst(Ast* parent);
-    ExpressionAst* destination;
-    QList<ExpressionAst*> values;
-    bool newline;
-};
-
 class KDEVPYTHONPARSER_EXPORT PassAst : public StatementAst {
 public:
     PassAst(Ast* parent);
 };
 
+class KDEVPYTHONPARSER_EXPORT NonlocalAst : public StatementAst {
+public:
+    NonlocalAst(Ast* parent);
+};
+
 
 /** Expression classes **/
 class KDEVPYTHONPARSER_EXPORT ExpressionAst : public Ast {
@@ -417,6 +446,18 @@ public:
     CallAst* belongsToCall;
 };
 
+class KDEVPYTHONPARSER_EXPORT AwaitAst : public ExpressionAst {
+public:
+    AwaitAst(Ast* parent);
+    ExpressionAst* value;
+};
+
+class KDEVPYTHONPARSER_EXPORT YieldFromAst : public ExpressionAst {
+public:
+    YieldFromAst(Ast* parent);
+    ExpressionAst* value;
+};
+
 class KDEVPYTHONPARSER_EXPORT BooleanOperationAst : public ExpressionAst {
 public:
     BooleanOperationAst(Ast* parent);
@@ -441,6 +482,7 @@ public:
             case Python::Ast::OperatorLeftShift: return QLatin1String("__lshift__");
             case Python::Ast::OperatorMod: return QLatin1String("__mod__");
             case Python::Ast::OperatorMult: return QLatin1String("__mul__");
+            case Python::Ast::OperatorMatMult: return QLatin1String("__matmul__");
             case Python::Ast::OperatorPow: return QLatin1String("__pow__");
             case Python::Ast::OperatorRightShift: return QLatin1String("__rshift__");
             case Python::Ast::OperatorSub: return QLatin1String("__sub__");
@@ -551,6 +593,12 @@ public:
     QString value;
 };
 
+class KDEVPYTHONPARSER_EXPORT BytesAst : public ExpressionAst {
+public:
+    BytesAst(Ast* parent);
+    QString value;
+};
+
 class KDEVPYTHONPARSER_EXPORT YieldAst : public ExpressionAst {
 public:
     YieldAst(Ast* parent);
@@ -564,14 +612,24 @@ public:
     ExpressionAst::Context context;
 };
 
+class KDEVPYTHONPARSER_EXPORT NameConstantAst : public ExpressionAst {
+public:
+    NameConstantAst(Ast* parent);
+    enum NameConstantTypes {
+        False,
+        True,
+        None,
+        Invalid // should not happen
+    };
+    NameConstantTypes value;
+};
+
 class KDEVPYTHONPARSER_EXPORT CallAst : public ExpressionAst {
 public:
     CallAst(Ast* parent);
     ExpressionAst* function;
     QList<ExpressionAst*> arguments;
     QList<KeywordAst*> keywords;
-    ExpressionAst* keywordArguments;
-    ExpressionAst* starArguments;
 };
 
 class KDEVPYTHONPARSER_EXPORT AttributeAst : public ExpressionAst {
@@ -580,6 +638,7 @@ public:
     ExpressionAst* value;
     Identifier* attribute;
     ExpressionAst::Context context;
+    int depth;
 };
 
 class KDEVPYTHONPARSER_EXPORT SubscriptAst : public ExpressionAst {
@@ -590,6 +649,13 @@ public:
     ExpressionAst::Context context;
 };
 
+class KDEVPYTHONPARSER_EXPORT StarredAst : public ExpressionAst {
+public:
+    StarredAst(Ast* parent);
+    ExpressionAst* value;
+    ExpressionAst::Context context;
+};
+
 class KDEVPYTHONPARSER_EXPORT ListAst : public ExpressionAst {
 public:
     ListAst(Ast* parent);
@@ -636,15 +702,21 @@ public:
 };
 
 /** Independent classes **/
+
+class KDEVPYTHONPARSER_EXPORT ArgAst : public Ast {
+public:
+    ArgAst(Ast* parent);
+    Identifier* argumentName;
+    ExpressionAst* annotation;
+};
+
 class KDEVPYTHONPARSER_EXPORT ArgumentsAst : public Ast {
 public:
     ArgumentsAst(Ast* parent);
-    QList<ExpressionAst*> arguments;
+    QList<ArgAst*> arguments;
     QList<ExpressionAst*> defaultValues;
-    Identifier* vararg;
-    Identifier* kwarg;
-    int arg_lineno, arg_col_offset;
-    int vararg_lineno, vararg_col_offset;
+    ArgAst* vararg;
+    ArgAst* kwarg;
 };
 
 class KDEVPYTHONPARSER_EXPORT KeywordAst : public Ast {
@@ -666,7 +738,7 @@ class KDEVPYTHONPARSER_EXPORT ExceptionHandlerAst : public Ast {
 public:
     ExceptionHandlerAst(Ast* parent);
     ExpressionAst* type;
-    ExpressionAst* name;
+    Identifier* name;
     QList<Ast*> body;
 };
 
