@@ -25,13 +25,30 @@
 #include <QLineEdit>
 #include <QCheckBox>
 
-class PEP8KCModule : public KCModule
-{
-public:
-    PEP8KCModule( QWidget* parent, const QVariantList& args = QVariantList() );
-    virtual void save();
-    virtual ~PEP8KCModule();
+#include <KPluginFactory>
 
+#include <interfaces/configpage.h>
+
+class PEP8KCModule : public KDevelop::ConfigPage
+{
+Q_OBJECT
+public:
+    PEP8KCModule(KDevelop::IPlugin* plugin, QWidget* parent = nullptr);
+    ~PEP8KCModule();
+
+    virtual QString name() const override;
+    virtual QString fullName() const override;
+    virtual QIcon icon() const override;
+
+
+    // TODO: use KConfigXT instead (this will additionally allow removing apply/reset/defaults)
+    static QString pep8Path(const KConfigGroup& group);
+    static bool isPep8Enabled(const KConfigGroup& group);
+    static QString pep8Arguments(const KConfigGroup& group);
+
+    virtual void apply() override;
+    virtual void reset() override;
+    virtual void defaults() override;
 private:
     KConfigGroup configGroup;
     QLineEdit* pep8url;
