@@ -22,16 +22,12 @@
 
 #include <QMutexLocker>
 
-#include <KDebug>
-#include <KComponentData>
-#include <KStandardDirs>
 #include <KPluginFactory>
 #include <KPluginLoader>
 
 #include <interfaces/icore.h>
 #include <interfaces/ilanguagecontroller.h>
 #include <interfaces/iplugincontroller.h>
-#include <interfaces/ilanguage.h>
 #include <interfaces/idocument.h>
 #include <interfaces/isourceformatter.h>
 #include <interfaces/idocumentcontroller.h>
@@ -55,16 +51,15 @@
 #include "codegen/refactoring.h"
 #include "codegen/correctionfilegenerator.h"
 #include "kdevpythonversion.h"
+#include "pep8kcm/kcm_pep8.h"
+#include "docfilekcm/kcm_docfiles.h"
+
+#include <QDebug>
+#include "pythondebug.h"
 
 using namespace KDevelop;
 
-K_PLUGIN_FACTORY( KDevPythonSupportFactory, registerPlugin<Python::LanguageSupport>(); )
-
-K_EXPORT_PLUGIN(KDevPythonSupportFactory(
-    KAboutData("kdevpythonsupport", "kdevpython", ki18n("Python Support"),
-               KDEVPYTHON_VERSION_STR, ki18n("Support for the Python Scripting Language"), KAboutData::License_GPL)
-    .addAuthor(ki18n("Sven Brauch"), ki18n("Author"), "svenbrauch@googlemail.com", "")
-))
+K_PLUGIN_FACTORY_WITH_JSON( KDevPythonSupportFactory, "kdevpythonsupport.json", registerPlugin<Python::LanguageSupport>(); )
 
 namespace Python
 {
@@ -75,7 +70,7 @@ KDevelop::ContextMenuExtension LanguageSupport::contextMenuExtension(KDevelop::C
     ContextMenuExtension cm;
     EditorContext *ec = dynamic_cast<KDevelop::EditorContext *>(context);
 
-    if (ec && ICore::self()->languageController()->languagesForUrl(ec->url()).contains(language())) {
+    if (ec && ICore::self()->languageController()->languagesForUrl(ec->url()).contains(this)) {
         // It's a Python file, let's add our context menu.
         m_refactoring->fillContextMenu(cm, context);
         TypeCorrection::self().doContextMenu(cm, context);
@@ -84,12 +79,13 @@ KDevelop::ContextMenuExtension LanguageSupport::contextMenuExtension(KDevelop::C
 }
 
 LanguageSupport::LanguageSupport( QObject* parent, const QVariantList& /*args*/ )
-    : KDevelop::IPlugin( KDevPythonSupportFactory::componentData(), parent )
+    : KDevelop::IPlugin("pythonlanguagesupport", parent )
     , KDevelop::ILanguageSupport()
     , m_highlighting( new Highlighting( this ) )
     , m_refactoring( new Refactoring( this ) )
 {
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::ILanguageSupport )
+    KDEV_USE_EXTENSION_INTERFACE( KDevelop::ILanguageCheckProvider )
 
     m_self = this;
 
@@ -105,7 +101,7 @@ LanguageSupport::LanguageSupport( QObject* parent, const QVariantList& /*args*/
 
 void LanguageSupport::documentOpened(IDocument* doc)
 {
-    if ( ! ICore::self()->languageController()->languagesForUrl(doc->url()).contains(language()) ) {
+    if ( ! ICore::self()->languageController()->languagesForUrl(doc->url()).contains(this) ) {
         // not a python file
         return;
     }
@@ -124,12 +120,7 @@ LanguageSupport::~LanguageSupport()
 
 KDevelop::ParseJob *LanguageSupport::createParseJob( const IndexedString& url )
 {
-    if ( enabledForFile(url.toUrl()) ) {
-        return new ParseJob(url, this);
-    }
-    else {
-        return 0;
-    }
+    return new ParseJob(url, this);
 }
 
 QString LanguageSupport::name() const
@@ -142,42 +133,6 @@ LanguageSupport* LanguageSupport::self()
     return m_self;
 }
 
-bool LanguageSupport::enabledForFile(const KUrl& url)
-{
-    // This is a bit more general than it would need to be,
-    // but that way we can have the same code for both branches.
-    QList< ILanguage* > enabledLanguages = ICore::self()->languageController()->languagesForUrl(url);
-    const QString& name = LanguageSupport::self()->name();
-    static const QString otherName = ( name == "Python3" ? "Python" : "Python3" );
-    bool haveBoth = false;
-    foreach ( const ILanguage* lang, enabledLanguages ) {
-        if ( lang->name() == otherName ) {
-            // both py2 and py3 plugins are installed
-            haveBoth = true;
-        }
-    }
-    if ( ! haveBoth ) {
-        // If only one of the plugins is installed, use that.
-        return true;
-    }
-
-    // Otherwise, both plugins are installed, so check if there's a choice for this session.
-    KDevelop::ISession* activeSession = KDevelop::ICore::self()->activeSession();
-    if ( activeSession ) {
-        KConfigGroup group(activeSession->config()->group("python"));
-        const QString& version = group.readEntry("languageVersion", "Python 3");
-        if ( ( version == "Python 3" && name == "Python3" ) || ( version == "Python 2" && name == "Python" ) ) {
-            // this plugin is the right one, the other one will disable itself
-            return true;
-        }
-    }
-    else {
-        // no session, treat this as a py3 file
-        return name == "Python3";
-    }
-    return false;
-}
-
 SourceFormatterItemList LanguageSupport::sourceFormatterItems() const
 {
     SourceFormatterStyle autopep8("pep8ify");
@@ -187,17 +142,17 @@ SourceFormatterItemList LanguageSupport::sourceFormatterItems() const
                                "def function(arg,*vararg,**kwargs): return arg+kwarg[0]\nfunction(3, 5, 7)");
     using P = SourceFormatterStyle::MimeHighlightPair;
     autopep8.setMimeTypes(SourceFormatterStyle::MimeList{ P{"text/x-python", "Python"} });
-    autopep8.setContent("/usr/bin/pep8ify -w $TMPFILE");
+    QString pep8ifyPath = QStandardPaths::findExecutable("pep8ify");
+    if (pep8ifyPath.isEmpty()) {
+        // TODO: proper error handling/user notification
+        qWarning() << "Could not find the pep8ify executable";
+        pep8ifyPath = "/usr/bin/pep8ify";
+    }
+    autopep8.setContent(pep8ifyPath + " -w $TMPFILE");
 
     return SourceFormatterItemList{SourceFormatterStyleItem{"customscript", autopep8}};
 }
 
-KDevelop::ILanguage *LanguageSupport::language()
-{
-    kDebug() << core()->languageController()->language( name() );
-    return core()->languageController()->language( name() );
-}
-
 KDevelop::ICodeHighlighting* LanguageSupport::codeHighlighting() const
 {
     return m_highlighting;
@@ -213,6 +168,27 @@ ILanguageSupport::WhitespaceSensitivity LanguageSupport::whitespaceSensititivy()
     return ILanguageSupport::IndentOnly;
 }
 
+QList<ILanguageCheck*> LanguageSupport::providedChecks()
+{
+    return {};
+}
+
+int LanguageSupport::configPages() const
+{
+    return 2;
+}
+
+KDevelop::ConfigPage* LanguageSupport::configPage(int number, QWidget* parent)
+{
+    if (number == 0) {
+        return new PEP8KCModule(this, parent);
+    } else if (number == 1) {
+        return new DocfilesKCModule(this, parent);
+    }
+    return nullptr;
+}
+
+
 }
 
 #include "pythonlanguagesupport.moc"
